//-----------------------------------------------------------------------------
// File: GLApp.cpp
//
// Desc: Application class for the imagire's OprnGL samples framework library.
//-----------------------------------------------------------------------------
#define STRICT
#include <windows.h>
#include <tchar.h>
#include <mmsystem.h>

#include "GLApp.h"
//-----------------------------------------------------------------------------
// Global access to the app (needed for the global WndProc())
//-----------------------------------------------------------------------------
static CD3DApplication* g_pD3DApp = NULL;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

//-----------------------------------------------------------------------------
// Name: WndProc()
// Desc: Static msg handler which passes messages to the application class.
//-----------------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    return g_pD3DApp->MsgProc( hWnd, uMsg, wParam, lParam );
}

//-------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: AvP[ṼRXgN^
//-------------------------------------------------------------
CD3DApplication::CD3DApplication()
{
	g_pD3DApp = this;
	m_bGL = FALSE;
    m_bDeviceObjectsInited = false;
    m_bDeviceObjectsRestored = false;

	m_dwCreationWidth           = 300;
    m_dwCreationHeight          = 300;
    m_strWindowTitle            = TEXT( "My Application" );

	m_fTime = 0;
	m_fElapsedTime = 0;
}
//-------------------------------------------------------------
HRESULT CD3DApplication::Create( HINSTANCE hInst, int nCmdShow )
{
	// O[o XgO܂
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style		= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "OpenGL Application";
	wcex.hIconSm		= NULL;
	RegisterClassEx( &wcex );

	// AvP[V̏s܂:
   m_hWnd = CreateWindow( wcex.lpszClassName, m_strWindowTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, m_dwCreationWidth, m_dwCreationHeight, NULL, NULL, hInst, NULL);
   if( !m_hWnd ) return FALSE;

    // Save window properties
    m_dwWindowStyle = GetWindowLong( m_hWnd, GWL_STYLE );
    GetWindowRect( m_hWnd, &m_rcWindowBounds );
    GetClientRect( m_hWnd, &m_rcWindowClient );

	this->OneTimeSceneInit();

   if(FAILED(Initialize3DEnvironment())){
		return FALSE;
	}
	m_bGL = TRUE;

	ShowWindow( m_hWnd, nCmdShow );
	UpdateWindow( m_hWnd );

   return TRUE;
}
//OpenGL_OReLXg̍쐬
//sNZtH[}bg̐ݒ
HRESULT CD3DApplication::Initialize3DEnvironment()
{
	HGLRC hRC;
	HDC hDC;
	int pixelformat;

    static PIXELFORMATDESCRIPTOR pfd = {
	sizeof(PIXELFORMATDESCRIPTOR),		//̍\̂̃TCY
	2,									//OpenGLo[W
	PFD_DRAW_TO_WINDOW |				//WindowX^C
		PFD_SUPPORT_OPENGL |			//OpenGL
		PFD_DOUBLEBUFFER,				//_uobt@gp\
	PFD_TYPE_RGBA,						//RGBAJ[
	24,									//F
	0, 0,								//RGBÃrbgƃVtgݒ		
	0, 0,								//G
	0, 0,								//B
	0, 0,								//A
	0,									//AL[Vobt@
	0, 0, 0, 0, 						//RGBAAL[Vobt@
	16,									//Zobt@	
	0,									//XeVobt@
	0,									//gpȂ
	PFD_MAIN_PLANE,						//C[^Cv
	0,									//\
	0, 0, 0								//C[}XN̐ݒEgp
    };

	//foCXReLXg
	if (!(hDC=GetDC(m_hWnd))){
		MessageBox(m_hWnd,"Getting HDC Failed....","ERROR",MB_OK);
		return NULL;
	}
	//sNZtH[}bg̎w
    if ( (pixelformat = ChoosePixelFormat(hDC, &pfd)) == 0 ){
        MessageBox(m_hWnd, "ChoosePixelFormat Failed....", "Error", MB_OK);
        return NULL;
    }
	//sNZtH[}bg̐ݒ
    if (SetPixelFormat(hDC, pixelformat, &pfd) == FALSE){
        MessageBox(m_hWnd, "SetPixelFormat Failed....", "Error", MB_OK);
        return NULL;
    }
	//OpenGL_OReLXg̍쐬
	if (!(hRC=wglCreateContext(hDC))){
		MessageBox(NULL,"Creating HGLRC Failed....","ERROR",MB_OK);
		return NULL;
	}
	ReleaseDC(m_hWnd,hDC);
	
	if(NULL==hRC) return -1;

	//JgReLXg
	hDC = GetDC(m_hWnd);
	wglMakeCurrent(hDC,hRC);


	// Depth eXg̏ݒ
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);

	glEnable(GL_CULL_FACE);
	glCullFace(GL_FRONT);
	//NCAg̈̓hԂF̐ݒ
	glClearColor( 0.0, 0.0, 0.0, 0.0 );
	//fvXobt@NAl̐ݒ
	glClearDepth( 1.0 );

    // AvP[ṼfoCXɈˑIuWFNg
    HRESULT hr = InitDeviceObjects();
    if( FAILED(hr) )
    {
        DeleteDeviceObjects();
    }
    else
    {
        m_bDeviceObjectsInited = true;
        hr = RestoreDeviceObjects();
        if( FAILED(hr) )
        {
            InvalidateDeviceObjects();
        }
        else
        {
            m_bDeviceObjectsRestored = true;
            return S_OK;
        }
    }


	return S_OK;
}
//-----------------------------------------------------------------------------
// Name: Reset3DEnvironment()
// Desc:
//-----------------------------------------------------------------------------
HRESULT CD3DApplication::Reset3DEnvironment()
{
    HRESULT hr;

	//@uq`l̃IuWFNgJ
    if( m_bDeviceObjectsRestored )
    {
        m_bDeviceObjectsRestored = false;
        InvalidateDeviceObjects();
    }

    // AvP[ṼfoCXɈˑIuWFNg
    hr = RestoreDeviceObjects();
    if( FAILED(hr) )
    {
        InvalidateDeviceObjects();
        return hr;
    }

    return S_OK;
}
//-------------------------------------------------------------
//
//
//-------------------------------------------------------------
void    CD3DApplication::Cleanup3DEnvironment()
{
    if( m_bGL )
    {
        if( m_bDeviceObjectsRestored )
        {
            m_bDeviceObjectsRestored = false;
            InvalidateDeviceObjects();
        }
        if( m_bDeviceObjectsInited )
        {
            m_bDeviceObjectsInited = false;
            DeleteDeviceObjects();
        }

		//OpenGL_OReLXg
		//foCXReLXg擾
		HGLRC hRC = wglGetCurrentContext();
		HDC hDC = wglGetCurrentDC();

		//JgReLXg
		wglMakeCurrent(NULL, NULL);
		if(!hRC){
			//OpenGL_OReLXgȂ
			MessageBox(m_hWnd,"Release HGLRC Failed....","ERROR",MB_OK);
		}
		if(hRC){
			//
			wglDeleteContext(hRC);
		}
		if(!hDC){
			//foCXReLXgȂ
			MessageBox(m_hWnd,"Release HDC Failed....","ERROR",MB_OK);
		}
		if(hDC){
			//
			ReleaseDC(m_hWnd, hDC);
		}

        m_bGL = FALSE;
    }
}
//-------------------------------------------------------------
INT CD3DApplication::Run()
{
	MSG msg;

	// C bZ[W [v:
	while( GetMessage(&msg, NULL, 0, 0) ) 
	{
		if( !TranslateAccelerator (msg.hwnd, NULL, &msg) ) 
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );

            if( FAILED( Render3DEnvironment() ) )
                SendMessage( m_hWnd, WM_CLOSE, 0, 0 );
		}
	}

	return msg.wParam;
}




// Name: Render3DEnvironment()
// Desc: Draws the scene.
//-----------------------------------------------------------------------------
HRESULT CD3DApplication::Render3DEnvironment()
{
    HRESULT hr;

    // Ԃv
	FLOAT time = 0.001f*(FLOAT)timeGetTime();
	m_fElapsedTime = (FLOAT)(time-m_fTime);
	m_fTime = time;

	// ACh֐܂킷
    if( FAILED( hr = FrameMove() ) )
        return hr;

    // `֐܂킷
    if( FAILED( hr = Render() ) )
        return hr;

//    UpdateStats();

	//_uobt@
	HDC hDC = wglGetCurrentDC();
	SwapBuffers(hDC);

	return S_OK;
}




//
//  ֐: WndProc(HWND, unsigned, WORD, LONG)
//
//  pr: C EBhẼbZ[W܂B
//
//  WM_COMMAND	- AvP[V j[̏
//  WM_PAINT	- C EBhE̕`
//  WM_DESTROY	- IbZ[W̒ʒmƃ^[
//
//
LRESULT CD3DApplication::MsgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;

	switch( message ) 
	{
    	case WM_CREATE:// 
			break;
		case WM_PAINT:// `
            if( m_bGL && //!m_bActive && m_bWindowed &&
				m_bDeviceObjectsInited && m_bDeviceObjectsRestored ){
				Render();
				//_uobt@
				HDC hDC = wglGetCurrentDC();
				SwapBuffers(hDC);
			}
			break;
		case WM_SIZE:
	        HandlePossibleSizeChange();			//TCYOpenGL̏
            break;
		case WM_DESTROY:// S
			Cleanup3DEnvironment();
			FinalCleanup();
			PostQuitMessage( 0 );
			break;

		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// j[Ỉ:
//			switch( wmId ) 
//			{
//				case IDM_EXIT:
//				   DestroyWindow( hWnd );
//				   break;
//				default:
//				   return DefWindowProc( hWnd, message, wParam, lParam );
//			}
			break;
		default:
			return DefWindowProc( hWnd, message, wParam, lParam );
   }
   return 0;
}
//-----------------------------------------------------------------------------
// Name: HandlePossibleSizeChange()
// Desc: Reset the device if the client area size has changed.
//-----------------------------------------------------------------------------
HRESULT CD3DApplication::HandlePossibleSizeChange()
{
    RECT rcClientOld;
    rcClientOld = m_rcWindowClient;

    // Update window properties
    GetWindowRect( m_hWnd, &m_rcWindowBounds );
    GetClientRect( m_hWnd, &m_rcWindowClient );

   	m_mysdBackBuffer.Width  = m_rcWindowClient.right - m_rcWindowClient.left;
	m_mysdBackBuffer.Height = m_rcWindowClient.bottom - m_rcWindowClient.top;

	if( rcClientOld.right - rcClientOld.left !=
        m_rcWindowClient.right - m_rcWindowClient.left ||
        rcClientOld.bottom - rcClientOld.top !=
        m_rcWindowClient.bottom - m_rcWindowClient.top)
    {
        if( m_bGL )	Reset3DEnvironment();
    }

	//ɂăr[|[gݒ肷
	glViewport(0, 0, m_mysdBackBuffer.Width, m_mysdBackBuffer.Height);

	return S_OK;
}
